"use strict";
const utils_1 = require("./utils");
const jaul = require("jaul");
const settings = require("setmeup").settings;
class Logger {
    static argsCleaner(obj, index) {
        const max = settings.logger.maxDepth;
        if ((0, utils_1.isArray)(obj)) {
            for (let i = 0; i < obj.length; i++) {
                if (index >= max) {
                    obj[i] = "...";
                }
                else if ((0, utils_1.isFunction)(obj[i])) {
                    obj[i] = "[Function]";
                }
                else {
                    Logger.argsCleaner(obj[i], index + 1);
                }
            }
        }
        else if ((0, utils_1.isObject)(obj)) {
            let keys = Object.keys(obj);
            for (let key of keys) {
                let value = obj[key];
                try {
                    if (index >= max) {
                        obj[key] = "...";
                    }
                    else if (settings.logger.obfuscateFields && settings.logger.obfuscateFields.indexOf(key) >= 0) {
                        obj[key] = "****";
                    }
                    else if (settings.logger.maskFields && settings.logger.maskFields[key]) {
                        let maskedValue;
                        if ((0, utils_1.isObject)(value)) {
                            maskedValue = value.value || value.text || value.data || value.toString();
                        }
                        else {
                            maskedValue = value.toString();
                        }
                        obj[key] = jaul.data.maskString(maskedValue, "*", settings.logger.maskFields[key]);
                    }
                    else if (settings.logger.removeFields && settings.logger.removeFields.indexOf(key) >= 0) {
                        delete obj[key];
                    }
                    else if ((0, utils_1.isArray)(value)) {
                        for (let i = 0; i < value.length; i++) {
                            Logger.argsCleaner(value[i], index + 1);
                        }
                    }
                    else if ((0, utils_1.isFunction)(value)) {
                        obj[key] = "[Function]";
                    }
                    else if ((0, utils_1.isObject)(value)) {
                        Logger.argsCleaner(value, index + 1);
                    }
                }
                catch (ex) {
                    delete obj[key];
                    obj[key] = "[Unreadable]";
                }
            }
        }
    }
    static clean(args) {
        let result = [];
        for (let arg of args) {
            if ((0, utils_1.isError)(arg)) {
                result.push(arg.toString());
            }
            else if ((0, utils_1.isObject)(arg) || (0, utils_1.isArray)(arg)) {
                let cloned = JSON.parse(JSON.stringify(arg, null, 0));
                Logger.argsCleaner(cloned, 0);
                result.push(cloned);
            }
            else if ((0, utils_1.isFunction)(arg)) {
                result.push("[Function]");
            }
            else {
                result.push(arg);
            }
        }
        return result;
    }
}
module.exports = Logger;
