"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isString = exports.isFunction = exports.isError = exports.isArray = exports.isPlainObject = exports.isObject = exports.getTag = void 0;
const getTag = (value) => {
    const toString = Object.prototype.toString;
    if (value == null) {
        return value === undefined ? "[object Undefined]" : "[object Null]";
    }
    return toString.call(value);
};
exports.getTag = getTag;
const isObject = (value) => {
    return typeof value === "object" && value !== null;
};
exports.isObject = isObject;
const isPlainObject = (value) => {
    if (!(0, exports.isObject)(value) || (0, exports.getTag)(value) != "[object Object]") {
        return false;
    }
    if (Object.getPrototypeOf(value) === null) {
        return true;
    }
    let proto = value;
    while (Object.getPrototypeOf(proto) !== null) {
        proto = Object.getPrototypeOf(proto);
    }
    return Object.getPrototypeOf(value) === proto;
};
exports.isPlainObject = isPlainObject;
const isArray = (value) => {
    return value && Array.isArray(value);
};
exports.isArray = isArray;
const isError = (value) => {
    if (!(0, exports.isObject)(value)) {
        return false;
    }
    const tag = (0, exports.getTag)(value);
    return tag == "[object Error]" || tag == "[object DOMException]" || (typeof value.message === "string" && typeof value.name === "string" && !(0, exports.isPlainObject)(value));
};
exports.isError = isError;
const isFunction = (value) => {
    return typeof value === "function";
};
exports.isFunction = isFunction;
const isString = (value) => {
    const type = typeof value;
    return type === "string" || (type === "object" && value != null && !Array.isArray(value) && (0, exports.getTag)(value) == "[object String]");
};
exports.isString = isString;
